from datetime import datetime, UTC
import logging
from json import JSONDecodeError

from httpx import AsyncClient
from sqlmodel import SQLModel, Field

from oidc_test.registry import (
    Resource,
    ProcessResult,
    ProcessError,
)
from oidc_test.models import User

from .bs_generator import sentence

logger = logging.getLogger("demo resource provider")


class Public(Resource):
    name: str = "Public"
    is_public: bool = True

    async def process(self, user: User | None, resource_id: str | None = None):
        return ProcessResult(msg="This is a public resource")


class Authenticated(Resource):
    name: str = "Authentication only required"

    async def process(
        self, user: User | None, resource_id: str | None = None
    ) -> ProcessResult:
        return ProcessResult(msg="Only authenticated users can see this")


class ProtectedByFoorole(Resource):
    name: str = "Protected by foorole"
    role_required: str | None = "foorole"

    async def process(
        self, user: User | None, resource_id: str | None = None
    ) -> ProcessResult:
        return ProcessResult(msg="Only users with foorole can see this")


class ProtectedByBarrole(Resource):
    name: str = "Protected by barrole"
    role_required: str | None = "barrole"

    async def process(
        self, user: User | None, resource_id: str | None = None
    ) -> ProcessResult:
        return ProcessResult(msg="Only users with barrole can see this")


class Time(Resource):
    name: str = "Time (UTC)"
    scope_required: str | None = "get:time"

    async def process(
        self, user: User | None, resource_id: str | None = None
    ) -> ProcessResult:
        return ProcessResult(time=datetime.now(UTC).strftime("%c %Z"))


class BS(Resource):
    name: str = "BS"
    scope_required: str | None = "get:bs"

    async def process(
        self, user: User | None, resource_id: str | None = None
    ) -> ProcessResult:
        return ProcessResult(bs=sentence.getString())
