from oidc_test.registry import Resource, ProcessResult, ProcessError


from datetime import datetime
import logging

from sqlmodel import SQLModel, Field

from oidc_test.models import User

logger = logging.getLogger("voting")


class Petition(SQLModel):
    id: int | None = Field(default=None, primary_key=True)
    title: str
    text: str
    url: str | None = None


class Signature(SQLModel):
    petition_id: int = Field(foreign_key="petition.id")
    signatory_id: str = Field(foreign_key="user.id")
    timestamp: datetime


class Demo(Resource):
    name: str = "Sign a petition"
    scope_required: str | None = "sign:petition"
    default_resource_id: str | None = "1"

    async def process(
        self, user: User | None, resource_id: str | None = None
    ) -> ProcessResult:
        if user is None:
            raise ProcessError("No user")
        if resource_id is None:
            raise ProcessError("No petition")
        # assert petition.id is not None
        signature = Signature(
            petition_id=int(resource_id),
            signatory_id=user.sub,
            timestamp=datetime.now(),
        )
        assert signature is not None
        logger.info(
            f"petition {resource_id} signed by {user.name} ({user.sub}, auth by {user.auth_provider_id})"
        )
        logger.warning("TODO!")
        ...
        return ProcessResult(
            msg=f"Thanks {user.name} for signing the petition!",
            comment=f"For information, your identidentity is ({user.sub}, authenticated by {user.auth_provider_id})",
        )
